;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; UCAsm Example Code
;; (c) Copyright 2007 Burnt Wasp and Figment Games
;;     All Rights Reserved.
;;
;; Filename:    CRC32.asm
;; Author:      Tom Bampton
;; Created:     1/4/2007
;; Purpose:
;;   CRC32 Example
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                include         "p8x32a.inc"
                include         "crc32.inc"

                clkmode         XTAL1_PLL16X
                xinfreq         6_000_000

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Startup Cog
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                cog             $20

StartupCog      ; Start new cogs
                cognew          CRC32Cog, @crcBufHead

                ; Stop this cog
                stopCurCog

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CRC32 cog args
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

crcBufHead      long            7
crcBufTail      long            0
crcBuffer       byte            "aaaaaa"
                res             byte 10
crcResult       long            $ffff_ffff

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CRC32 cog with cog memory based CRC table and hub based buffer
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Argument Structure format:
;
; Buffer Head   1 long
; Buffer Tail   1 long
; Buffer        16 bytes
; CRC32 Result  1 long

                cog
CRC32Cog

                ; Setup pointers
                mov             bufHead, PAR
                mov             bufTail, PAR
                mov             buffer, PAR

                add             bufTail, #4
                add             buffer, #8
                mov             crcResult, buffer
                add             crcResult, #16

                ; Check head and tail
:rdLoop         rdlong          d0, bufHead
                rdlong          d1, bufTail

                cmp             d0, d1                  wz
        if_z    jmp             #:rdLoop

                ; Read current crc value for later
                rdlong          crcValue, crcResult

                ; Read data
                mov             bufPtr, d1
                add             bufPtr, buffer
                rdbyte          d0, bufPtr

                ; Update tail
                add             d1, #1
                and             d1, #$0f
                wrlong          d1, bufTail

                ; Calculate CRC32

                ; index = (crcValue ^ byte) & $ff
                mov             index, crcValue
                xor             index, d0
                and             index, #$ff

                ; crcValue = crcTable[index] ^ (crcValue >> 8)
                mov             d0, #crcTable
                add             d0, index
                movs            $ + 1, d0
                mov             d1, $-$

                shr             crcValue, #8
                xor             crcValue, d1

                ; Write result
                wrlong          crcValue, crcResult

                ; Get next byte
                jmp             #:rdLoop


bufHead         res             1
bufTail         res             1
buffer          res             1
crcResult       res             1

bufPtr          res             1

d0              res             1
d1              res             1

crcValue        res             1
index           res             1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                fit             $ef
                org             $ef

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CRC32 Table (256 longs)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

crcTable        buildCRC32Tab

                fit
