;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Urinal Cake Assembler
;; (c) Copyright 2007 Burnt Wasp and Figment Games
;;     All Rights Reserved.
;;
;; Filename:    p8x32a.inc
;; Author:      Tom Bampton
;; Created:     15/2/2007
;; Purpose:
;;   Header for the Parallax Propeller P8X32A
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                ifndef          UCA_P8X32A_INC

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Version and Identification Constants
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Version number of this header file. Bump whenever the header is changed.
UCA_P8X32A_INC  equ             1

; Constant to identify the propeller chip. The value is meaningless.
P8X32A          equ             1

; Version of the propeller this header file supports
PROPELLER_VER   equ             1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Cog Registers
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PAR             equ             $1f0
CNT             equ             $1f1
INA             equ             $1f2
INB             equ             $1f3
OUTA            equ             $1f4
OUTB            equ             $1f5
DIRA            equ             $1f6
DIRB            equ             $1f7
CTRA            equ             $1f8
CTRB            equ             $1f9
FRQA            equ             $1fa
FRQB            equ             $1fb
PHSA            equ             $1fc
PHSB            equ             $1fd
VCFG            equ             $1fe
VSCL            equ             $1ff

; Read Only registers
                readonly        PAR, CNT, INA, INB

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CLKMODE register values
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

RCFAST          equ             %0_0_0_00_000
RCSLOW          equ             %0_0_0_00_001
XINPUT          equ             %0_0_0_00_010
XTAL1_PLL1X     equ             %0_1_1_01_011
XTAL1_PLL2X     equ             %0_1_1_01_100
XTAL1_PLL4X     equ             %0_1_1_01_101
XTAL1_PLL8X     equ             %0_1_1_01_110
XTAL1_PLL16X    equ             %0_1_1_01_111
XTAL1           equ             %0_0_1_01_010
XTAL2           equ             %0_0_1_10_010
XTAL3           equ             %0_0_1_11_010
XTAL2_PLL1X     equ             %0_1_1_10_011
XTAL2_PLL2X     equ             %0_1_1_10_100
XTAL2_PLL4X     equ             %0_1_1_10_101
XTAL2_PLL8X     equ             %0_1_1_10_110
XTAL2_PLL16X    equ             %0_1_1_10_111
XINPUT_PLL1X    equ             %0_1_1_00_011
XINPUT_PLL2X    equ             %0_1_1_00_100
XINPUT_PLL4X    equ             %0_1_1_00_101
XINPUT_PLL8X    equ             %0_1_1_00_110
XINPUT_PLL16X   equ             %0_1_1_00_111
XTAL3_PLL1X     equ             %0_1_1_11_011
XTAL3_PLL2X     equ             %0_1_1_11_100
XTAL3_PLL4X     equ             %0_1_1_11_101
XTAL3_PLL8X     equ             %0_1_1_11_110
XTAL3_PLL16X    equ             %0_1_1_11_111

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Macros
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; cognew       entry, parValue, writeID
;;
;; Start a new cog, optionally storing the ID for retrieval by the
;; getCogID macro.
;;
;; Note that you do not need to specify the @ for the code label, it is
;; added automatically.
;;
;; Arguments:
;;      entry           Long aligned hub address of cog entry point
;;      parValue        16 bit value for par register. Lower 2 bits are dropped.
;;      writeID         true to write the cog ID, false otherwise
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
cognew          macro           codeAddr, parValue
                local           cogWord

                coginit         cogWord         wr

                jmp             #cogWord + 1
cogWord         long            ((parValue & $fffc) << 16) | ((@codeAddr & $fffc) << 2) | %1_000
                end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; stopCurCog
;;
;; Stop the current cog. Uses 2 instructions and no temporary storage.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
stopCurCog      macro
                cogid           $
                cogstop         $ - 1
                end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                end             ; UCA_P8X32A_INC
